@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w -T
 #
-# $Header: /home/LDMS/cvsroot/LDMS/LDMS.pl,v 1.30 2000/11/28 18:27:33 jcl53 Exp $
+# $Header: /home/LDMS/cvsroot/LDMS/LDMS.pl,v 1.38 2000/12/01 02:13:45 jcl53 Exp $
 
 
 #==============================================================================
@@ -71,17 +71,22 @@ GetOptions('help|h|?' => sub {&showHelp(); exit},
 
 
 ## Get input filename, and validate, if necessary.
-if (scalar(@ARGV) > 0) {
+#*** NOTE TO SELF - Try to use STDIN if we don't get an input filename.
+if (scalar(@ARGV) > 0) {  # We should receive input file as an argument.
     $inputFilename = shift(@ARGV);
 }
-else {
+else {  # If we didn't get an input file, show help.
     &showHelp();
     print STDERR "No input file specified!\n";
     exit;
 }
-if (!(-r $inputFilename)) {
+if (!(-e $inputFilename)) {  # Check to see if the input file exists.
     &showHelp();
-    print STDERR "Input file \'$inputFilename\'is not readable.\n";
+    print STDERR "Input file \'$inputFilename\' not found!\n";
+}
+if (!(-r $inputFilename)) {  # Check to see if we can read input file.
+    &showHelp();
+    print STDERR "Input file \'$inputFilename\' is not readable.\n";
     exit;
 }
 
@@ -89,16 +94,16 @@ if (!(-r $inputFilename)) {
 ## Display help message, if required.
 if (scalar(@ARGV) > 0) {  # Show help if there's still junk at command line.
     &showHelp();
-    print STDERR "Bad parameter \'$ARGV[0]\'!\n";
+    print STDERR "Bad parameter(s) \'$ARGV[0]\'!\n";
     exit;
 }
 
 
 ## Set output filename.
-if ($opt_output ne "") {
+if ($opt_output ne "") {  # If passed an output filename, use that.
     $outputFilename = $opt_output;
 }
-else {
+else {  # Default output filename is input filename with .xml extension.
     $outputFilename = $inputFilename;
     $outputFilename =~ s/\.\w*$/\.xml/g;   
 }
@@ -162,16 +167,7 @@ $fh = &Initialize($outputFilename);
 ## Process the title, in blocks demarcated by -CITE-...
 for (my $i = 1; $i < scalar(@blockifiedTitle); $i++) {
 
-    if ($opt_verbose >= 1) {
-	if ((&getVLevel() > 0) && 
-	    ($opt_status > 0) && 
-	    (&getVLevel() <= $opt_status)) {
-
-	    print STDERR "Processing " . &getLevelLabel() . " " . &getHLevel() . ".\n";
-
-	}
-    }
-
+    # Grab current block.
     my $currentBlock = $blockifiedTitle[$i];
 
     # Get rid of the flanking -CITE- tags, and hose the leftover newline.
@@ -183,7 +179,8 @@ for (my $i = 1; $i < scalar(@blockifiedTitle); $i++) {
     # Only do everything else if the block is not empty.
     if (scalar(@blockArray) > 0) {
 
-	my $sectionLabel = "";  # What this title calls this division.
+	my $sectionLabel = "";   # What this title calls this division.
+	my $sequenceLabel = "";  # What this title numbered this division.
 	
 	# Grab the name of this structural division from -HEAD-.
 	my $headline = "";
@@ -197,6 +194,8 @@ for (my $i = 1; $i < scalar(@blockifiedTitle); $i++) {
 		if ($headline ne "") {
 		    my @section = split(/\s+/, $headline);
 		    $sectionLabel = $section[1];
+		    $sequenceLabel = $section[2];
+		    $sequenceLabel =~ s/\.$//;
 		}	      
 		last;
 	    }
@@ -206,17 +205,19 @@ for (my $i = 1; $i < scalar(@blockifiedTitle); $i++) {
 
 	# Begin new structural division.
 	if (&getVLevel() < 0) {  # Nothing in @path, assume top-level.
-	   &descendLevel($sectionLabel);
-	   &startStructdiv();
+	   &descendLevel($sectionLabel, $sequenceLabel);
+	   &startStructdiv();	   
 	}
 	elsif (&getLevelLabel() eq $sectionLabel) {  # Same level.
 	    &endStructdiv();
 	    &startStructdiv();
+	    &setSequenceLabel($sequenceLabel);	    
 	}
 	else {  # Go to proper level, or make a new one, if necessary.
 	    if (&labelInPath($sectionLabel) > 0) {
 		while (&getVLevel() >= 0) {
 		    if (&getLevelLabel() eq $sectionLabel) {
+			&setSequenceLabel($sequenceLabel);
 			&endStructdiv();
 			&startStructdiv();
 			last;
@@ -229,11 +230,23 @@ for (my $i = 1; $i < scalar(@blockifiedTitle); $i++) {
 		}
 	    }
 	    else {
-		&descendLevel($sectionLabel);
-		&startStructdiv();
+		&descendLevel($sectionLabel, $sequenceLabel);
+		&startStructdiv();	
 	    }
 	}
 
+	# Print out status, if necessary.
+	if ($opt_verbose >= 1) {
+	    if ((&getVLevel() > 0) && 
+		($opt_status > 0) && 
+		(&getVLevel() <= $opt_status)) {
+
+		&printProcessingLevel();
+		
+	    }
+	}
+	
+
 	# Start our TITLEDATA tag.
 	&BeginTag("TITLEDATA");
 	&WriteString("\n");
@@ -301,19 +314,32 @@ while (&getVLevel() >= 0) {
 &EndTag("LDMS");
 
 
+## If requested, tell user we're done processing and starting to print.
+if ($opt_verbose >= 1) {
+
+    print STDERR "Processing completed.  Writing to disk.\n"
+    
+}
+
+
 ## Finalize the XML document.
 &Finalize($fh);
 
 
 #==============================================================================
 # START STRUCTDIV TAG
-=pod
-
-=head2 startStructdiv()
-
-Produces a <STRUCTDIV> tag that starts at the current hierarchical level.
-
-=cut
+#
+# startStructdiv()
+#
+# Produces a <STRUCTDIV> tag that starts at the current hierarchical level.
+#
+# Preconditions:
+#   - Assumes state machine has useful data in it.
+#
+# Postconditions:
+#   - New STRUCTDIV tag added to output buffer.
+#   - State machine advances one sequence level.
+#
 
 sub startStructdiv() {
 
@@ -330,13 +356,18 @@ sub startStructdiv() {
 
 #==============================================================================
 # END STRUCTDIV TAG
-=pod
-
-=head2 endStructdiv()
-
-Produces a </STRUCTDIV> tag.
-
-=cut
+#
+# endStructdiv()
+#
+# Produces a </STRUCTDIV> tag.
+#
+# Preconditions:
+#   - None, really.  But you really shouldn't call this unless you've
+#     called startStructdiv() previously.
+#
+# Postconditions:
+#   - Adds a </STRUCTDIV> tag to the output buffer.
+#
 
 sub endStructdiv() {
 
@@ -348,13 +379,17 @@ sub endStructdiv() {
 
 #==============================================================================
 # FAST-FORWARD
-=pod
-
-=head2 fastForward()
-
-Fast-forwards to the next relevant block of text.
-
-=cut
+#
+# fastForward()
+#
+# Fast-forwards to the next relevant block of text.
+#
+# Preconditions:
+#   - Assumes useful data in @blockArray.
+#
+# Postconditions:
+#   - All data from current position to next dashline is sent to output buffer.
+#
 
 sub fastForward() {
     while ((scalar(@blockArray) > 0) && 
@@ -366,13 +401,19 @@ sub fastForward() {
 
 #==============================================================================
 # FIELD EXTRACTOR
-=pod
-
-=head2 getField()
-
-This takes everything up to the next dashline tag and puts it in the global variable @block.
-
-=cut
+#
+# getField()
+#
+# This takes everything up to the next dashline tag and puts it in the 
+# global variable @block.
+#
+# Preconditions:
+#   - Assumes useful data in @blockArray (note to self:  don't assume).
+#
+# Postconditions:
+#   - Returns an array of strings, one line per element, that represents
+#     all data up to the empty line before the next dashline.
+#
 
 sub getField() {
 
@@ -396,13 +437,17 @@ sub getField() {
 
 #==============================================================================
 # -CITE- HANDLER
-=pod
-
-=head2 markupCite()
-
-Takes lines in @blockArray and processes them with the LP module.
-
-=cut
+#
+# markupCite()
+#
+# Takes lines in @blockArray and processes them with the LP module.
+#
+# Preconditions:
+#   - Assumes @blockArray contains useful data.
+#
+# Postconditions:
+#   - The CITE block is marked up and sent to output buffer.
+#
 
 sub markupCite() {
 
@@ -447,13 +492,19 @@ sub markupCite() {
 
 #==============================================================================
 # -EXPCITE- HANDLER
-=pod
-
-=head2 markupExpcite()
-
-Extracts EXPCITE block from the current block, and processes it with the LDMS::LP module.
-
-=cut
+#
+# markupExpcite()
+#
+# Extracts EXPCITE block from the current block, and processes it with the 
+# LDMS::LP module.
+#
+# Preconditions:
+#   - Assumes that &getField() will provide useful data.
+#
+# Postconditions:
+#   - EXPCITE block is tagged and sent to output buffer.
+#   - DIVEXPCITE divisions are also tagged and sent to output buffer.
+#
 
 sub markupExpcite() {
     
@@ -489,13 +540,22 @@ sub markupExpcite() {
 
 #==============================================================================
 # GENERIC FIELD HANDLER
-=pod
-
-=head2 markupField($)
-
-Many fields can be handled alike.  This function processes the fields with the appropriate module, if any.  It determines what field it's working on by checking the parameter passed in for the name.
-
-=cut
+#
+# markupField($)
+#
+# Many fields can be handled alike.  This function processes the fields 
+# with the appropriate module, if any.  It determines what field it's 
+# working on by checking the parameter passed in for the name.
+#
+# Preconditions:
+#   - Requires a field name to be passed in.
+#   - Assumes that &getField() will provide useful data.
+#
+# Postconditions:
+#   - Field is tagged, if recognized, and sent to output buffer.
+#   - DataText, if any, is tagged, and sent to output buffer.
+#   - Any data that is not recognized is merely sent to output buffer as-is.
+#
 
 sub markupField($) {
 
@@ -521,9 +581,12 @@ sub markupField($) {
     # Further processing, dependent on expected type of contents.
     if ($usesDataText == 1) {  # For DataText blocks.
 	
-#	&processDataText(@block);
+	&BeginTag("DATATEXT");
+	&WriteString("\n");
 	&markUpTable(@block);
 	@block = ();
+	&EndTag("DATATEXT");
+	&WriteString("\n");
 
     }
     elsif ($fieldName eq "SOURCE") {
@@ -548,13 +611,20 @@ sub markupField($) {
 
 #==============================================================================
 # GENERIC FIELD PROCESSOR
-=pod
-
-=head2 processField($)
-
-This function does preliminary handling for generic fields.  It basically takes care of the dashline and then passes whatever's left on to the markupField($) subroutine.
-
-=cut
+#
+# processField($)
+#
+# This function does preliminary handling for generic fields.  It basically 
+# takes care of the dashline and then passes whatever's left on to the 
+# markupField($) subroutine.
+#
+# Preconditions:
+#   - Requires a field name to be passed in.
+#
+# Postconditions:
+#   - Dashline is removed.
+#   - Field is sent to &markupField() for actual processing.
+#
 
 sub processField($) {
 
@@ -576,13 +646,17 @@ sub processField($) {
 
 #==============================================================================
 # PRINT STATUS MESSAGES
-=pod
-
-=head2 showStatus()
-
-Prints a status message, if necessary.
-
-=cut
+#
+# showStatus()
+#
+# Prints a status message, if necessary.
+#
+# Preconditions:
+#   - $lineCounter contains useful data.
+#
+# Postconditions:
+#   - Status is printed by &printStatus().
+#
 
 sub showStatus() {
 
@@ -598,26 +672,30 @@ sub showStatus() {
 
 #==============================================================================
 # HELP FUNCTION
-=pod
-
-=head2 showHelp()
-
-Does exactly what it says.  Displays the current help text.
-
-=cut
+#
+# showHelp()
+#
+# Does exactly what it says.  Displays the current help text.
+#
+# Preconditions:
+#   - User did something wrong.
+#
+# Postconditions:
+#   - User is reeducated.
+#
 
 sub showHelp() {
 
     print STDERR <<"EndHelpText";
 
-Usage:  $0 -hvo <OUTPUT> [-] <FILE>
+Usage:  $0 -hvfo <OUTPUT> -s <DEPTH> [-] <FILE>
 
   Options:
-    -h, -?, --help                    Show this help message.
-    -v, --verbose                     Enable verbose output.
-    -o <OUTPUT>, --output <OUTPUT>    Write to file <OUTPUT>.
-    -f, --force                       Overwrite existing files.
-    -s, --status                      Set maximum status reporting depth.
+    -h, -?, --help                   Show this help message.
+    -v, --verbose                    Enable verbose output.
+    -o <OUTPUT>, --output <OUTPUT>   Write to file <OUTPUT>.  ('-' for STDOUT.)
+    -f, --force                      Overwrite existing files.
+    -s <DEPTH> , --status <DEPTH>    Set maximum status reporting depth.
 
 
 EndHelpText
@@ -641,15 +719,29 @@ B<./LDMS.pl> [I<OPTIONS>]... [I<FILE>]...
 
 Converts the given filename from 8-bit ASCII into XML, as per the LDMS DTD.
 
-B<-h, -?, --help>          Show this help message.
+=over 5
 
-B<-v, --verbose>           Enable verbose output.
+=item B<-h, -?, --help>          
 
-B<-o, --output [FILE]>     Write to FILE.
+Show this help message.
 
-B<-f, --force>             Overwrite existing files.
+=item B<-v, --verbose>           
 
-B<-s, --status>            Set maximum status reporting depth.
+Enable verbose output.
+
+=item B<-o, --output [FILE]>     
+
+Write to FILE.  (Use '-' for STDOUT.)
+
+=item B<-f, --force>             
+
+Overwrite existing files.
+
+=item B<-s, --status [DEPTH]>    
+
+Set maximum status reporting depth.
+
+=back
 
 =head1 AUTHORS
 
@@ -691,8 +783,14 @@ Brian Williams
 
 LDMS.dtd     Defines tags used in translating to XML.
 
+=head1 BUGS
+
+LDMS still does not achieve 100% accuracy.  Whether this is a bug in the code or a bug in the system that provides the code is left as an exercise for the motivated programmer.
+
 =head1 NOTES
 
+If you want to write to STDOUT, just supply '-' (without the single-quotes) as the output filename.
+
 As this script was written in Perl by several different developers, the motto "There's more than one way to do it!" may be quite evident in places.  Nevertheless, we've tried to make the code easy to modify and maintain...
 
 We hope we succeeded.
@@ -704,6 +802,32 @@ __END__;
 
 
 $Log: LDMS.pl,v $
+Revision 1.38  2000/12/01 02:13:45  jcl53
+Removed the redundant debugging code that was producing duplicate status messages.
+
+Revision 1.37  2000/11/30 20:23:06  jcl53
+Adds DATATEXT tags around the call to &markUpTable(@) so that it's guaranteed to keep tables inside LDMS tags.
+
+Revision 1.36  2000/11/30 16:08:00  jcl53
+Added more comments, particularly preconditions and postconditions.
+
+Revision 1.35  2000/11/29 06:11:32  jcl53
+Checks the -HEAD- field to figure out what the current section is numbered, according to the title.  Allows for more intuitive status messages.
+
+Revision 1.34  2000/11/29 05:07:57  jcl53
+Updated the documentation with a hint for sending output to STDOUT:  use the special filehandle '-'.
+
+Revision 1.33  2000/11/29 03:48:31  jcl53
+Amazing what a stray perldoc directive ('cut' in this case) can do to your happy, well-maintained code...
+
+Revision 1.32  2000/11/29 00:00:46  jcl53
+Uses the LDMS::Status module for printing status messages.
+
+Revision 1.31  2000/11/28 20:46:13  jcl53
+Now das Wunderscript only uses the LDMS::Footnote module if there are recognizable footnotes (or references) present within the current block.
+
+Currently, takes over an hour for Title 2.  Thank root that this isn't a problem for our requirements.
+
 Revision 1.30  2000/11/28 18:27:33  jcl53
 Added the status granularity setting.  Defaults to '2', or the 'Sec.' level in Title 1.
 
